package de.ozgcloud.xta.test.app.mapper;

import java.io.File;
import java.math.BigInteger;
import java.time.ZonedDateTime;
import java.util.GregorianCalendar;
import java.util.List;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;

import genv3.de.xoev.transport.xta.x211.ContentType;
import genv3.de.xoev.transport.xta.x211.GenericContentContainer;
import genv3.de.xoev.transport.xta.x211.LookupServiceResponse;
import genv3.de.xoev.transport.xta.x211.LookupServiceResultType;
import genv3.de.xoev.transport.xta.x211.TransportReport;
import genv3.eu.osci.ws.x2008.x05.transport.MsgBoxResponseType;
import genv3.eu.osci.ws.x2008.x05.transport.MsgStatusListType;
import genv3.eu.osci.ws.x2014.x10.transport.DeliveryAttributesType;
import genv3.eu.osci.ws.x2014.x10.transport.KeyCodeType;
import genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData;
import genv3.eu.osci.ws.x2014.x10.transport.PartyIdentifierType;
import genv3.eu.osci.ws.x2014.x10.transport.QualifierType;
import jakarta.activation.DataHandler;
import jakarta.activation.FileDataSource;
import lombok.SneakyThrows;
import org.apache.cxf.ws.addressing.AttributedURIType;
import org.mapstruct.*;

import de.ozgcloud.xta.test.app.model.*;

@Mapper(
		unmappedTargetPolicy = ReportingPolicy.ERROR,
		imports = { BigInteger.class }
)
public interface ResponseMapper {
	String IDENTIFIER_TYPE = "xoev";
	String MESSAGE_TYPE_LIST_URI = "urn:de:payloadSchema:elementName";
	String MESSAGE_TYPE_LIST_VERSION_ID = "1.0";
	String REQUEST_ID = "1";
	String NO_MESSAGES_REASON = "Keine Nachrichten gefunden.";
	XtaIdentifier SENDER_IDENTIFIER = XtaIdentifier.builder()
			.name("xta-test-server")
			.value("xta-test-server")
			.category("dbs:testumgebung")
			.build();

	@Mapping(target = "qualifier.service", source = "service")
	@Mapping(target = "qualifier.businessScenario", source = ".", qualifiedByName = "mapBusinessScenario")
	@Mapping(target = "qualifier.messageType", source = ".")
	@Mapping(target = "msgIdentification.messageID.value", source = "messageId")
	@Mapping(target = "originators.author.identifier", source = "authorIdentifier")
	@Mapping(target = "originators.sender.identifier", expression = "java( mapPartyIdentifierType(SENDER_IDENTIFIER) )")
	@Mapping(target = "destinations.reader.identifier", source = "readerIdentifier")
	@Mapping(target = "msgSize", source = "messageSize")
	@Mapping(target = "testMsg", ignore = true)
	@Mapping(target = "deliveryAttributes", expression = "java( createDefaultDeliveryAttributes(createNowXmlGregorianCalendar()) )")
	@Mapping(target = "messageProperties", ignore = true)
	MessageMetaData mapMessageMetaData(XtaMessageMetaData xtaMessageMetaData);

	@Named("mapBusinessScenario")
	default QualifierType.BusinessScenario mapBusinessScenario(XtaMessageMetaData xtaMessageMetaData) {
		return xtaMessageMetaData.businessScenarioListUri() != null
				? mapDefinedBusinessScenario(xtaMessageMetaData)
				: mapUndefinedBusinessScenario(xtaMessageMetaData);
	}

	@Mapping(target = "defined", expression = "java(null)")
	@Mapping(target = "undefined", source = "businessScenarioCode")
	QualifierType.BusinessScenario mapUndefinedBusinessScenario(XtaMessageMetaData xtaMessageMetaData);

	@Mapping(target = "defined", source = ".")
	@Mapping(target = "undefined", expression = "java(null)")
	QualifierType.BusinessScenario mapDefinedBusinessScenario(XtaMessageMetaData xtaMessageMetaData);

	@Mapping(target = "code", source = "businessScenarioCode")
	@Mapping(target = "name", source = "businessScenarioName")
	@Mapping(target = "listURI", source = "businessScenarioListUri")
	@Mapping(target = "listVersionID", source = "businessScenarioListVersionId")
	KeyCodeType mapKeyCodeType(XtaMessageMetaData xtaMessageMetaData);

	@Mapping(target = "code", source = "messageTypeCode")
	@Mapping(target = "name", ignore = true)
	@Mapping(target = "payloadSchema", source = "messageTypePayloadSchema")
	@Mapping(target = "listURI", constant = MESSAGE_TYPE_LIST_URI)
	@Mapping(target = "listVersionID", constant = MESSAGE_TYPE_LIST_VERSION_ID)
	QualifierType.MessageType mapMessageTypeQualifierType(XtaMessageMetaData xtaMessageMetaData);

	@Mapping(target = "type", constant = IDENTIFIER_TYPE)
	PartyIdentifierType mapPartyIdentifierType(XtaIdentifier value);

	@Mapping(target = "encryptedData", expression = "java(null)")
	@Mapping(target = "contentContainer.message", source = "messageFile")
	@Mapping(target = "contentContainer.attachment", source = "attachmentFiles")
	GenericContentContainer mapGenericContentContainerFromXtaMessage(XtaMessage xtaMessage);

	@Mapping(target = "value", source = "file")
	@Mapping(target = "filename", source = "name")
	@Mapping(target = "lang", source = "language")
	ContentType mapContentTypeFromXtaFile(XtaFile xtaFile);

	default DataHandler getDataHandlerFromFile(File file) {
		var dataSource = new FileDataSource(file);
		return new DataHandler(dataSource);
	}

	@Mapping(target = "delivery", source = ".")
	@Mapping(target = "initialFetch", source = ".")
	@Mapping(target = "initialSend", source = ".")
	@Mapping(target = "origin", source = ".")
	@Mapping(target = "notBefore", ignore = true)
	@Mapping(target = "obsoleteAfter", ignore = true)
	@Mapping(target = "reception", ignore = true)
	@Mapping(target = "serviceQuality", ignore = true)
	@Mapping(target = "receiptRequests", ignore = true)
	DeliveryAttributesType createDefaultDeliveryAttributes(XMLGregorianCalendar timestamp);

	@SneakyThrows
	default XMLGregorianCalendar createNowXmlGregorianCalendar() {
		return mapXmlGregorianCalendarFromZonedDateTime(ZonedDateTime.now());
	}

	@SneakyThrows
	default XMLGregorianCalendar mapXmlGregorianCalendarFromZonedDateTime(ZonedDateTime zonedDateTime) {
		GregorianCalendar gregorianCalendar = GregorianCalendar.from(zonedDateTime);
		return DatatypeFactory.newInstance().newXMLGregorianCalendar(gregorianCalendar);
	}

	@Mapping(target = "itemsPending", expression = "java( BigInteger.ZERO )")
	@Mapping(target = "msgBoxRequestID", source = "requestId")
	@Mapping(target = "noMessageAvailable", expression = "java( null )")
	MsgBoxResponseType mapMsgBoxResponseTypeFromRequestId(String requestId);

	@Mapping(target = "itemsPending", source = "pendingMessageCount")
	@Mapping(target = "msgBoxRequestID", constant = REQUEST_ID)
	@Mapping(target = "noMessageAvailable", source = "messages", qualifiedByName = "mapNoMessageAvailableReason")
	MsgBoxResponseType mapMsgBoxResponseType(XtaMessageMetaDataListing listing);

	@Named("mapNoMessageAvailableReason")
	default MsgBoxResponseType.NoMessageAvailable mapNoMessageAvailableReason(List<XtaMessageMetaData> messages) {
		return messages.isEmpty() ? createNoMessageAvailableWithReason(NO_MESSAGES_REASON) : null;
	}

	MsgBoxResponseType.NoMessageAvailable createNoMessageAvailableWithReason(String reason);

	@Condition
	default boolean pendingMessagesCountIsNotZero(BigInteger pendingMessageCount, @TargetPropertyName String targetPropertyName,
			@SourcePropertyName String sourcePropertyName) {
		if (targetPropertyName.equals("itemsPending")
				&& sourcePropertyName.equals("pendingMessageCount")) {
			return pendingMessageCount != null && !pendingMessageCount.equals(BigInteger.ZERO);
		}
		return true;
	}

	@Mapping(target = "messageMetaData", source = "messages")
	@Mapping(target = "msgAttributes", ignore = true)
	MsgStatusListType mapMsgStatusListType(XtaMessageMetaDataListing listing);

	@Mapping(target = "msgBoxRequestID", constant = REQUEST_ID)
	@Mapping(target = "itemsPending", ignore = true)
	@Mapping(target = "noMessageAvailable.reason", source = "errorString")
	MsgBoxResponseType mapMsgBoxResponseTypeError(String errorString);

	@Mapping(target = "isServiceAvailableValue.serviceIsAvailable", expression = "java( true )")
	@Mapping(target = "serviceType", source = "service")
	@Mapping(target = "reader.identifier", source = "readerIdentifier")
	@Mapping(target = "serviceParameter", ignore = true)
	LookupServiceResultType mapLookupServiceResultType(XtaLookupServiceRequest xtaLookupServiceRequest);

	@Mapping(target = "lookupServiceResultList.lookupServiceResult", expression = "java( List.of( mapLookupServiceResultType(xtaLookupServiceRequest) ) )")
	LookupServiceResponse mapLookupServiceResponse(XtaLookupServiceRequest xtaLookupServiceRequest);

	@Mapping(target = "messageMetaData", source = "messageMetaData")
	@Mapping(target = "reportTime", source = "reportTime")
	@Mapping(target = "XTAServerIdentity", constant = "xta-test-server")
	@Mapping(target = "messageStatus.status", expression = "java( BigInteger.valueOf(xtaTransportReport.status().ordinal()) )")
	@Mapping(target = "additionalReports", ignore = true)
	@Mapping(target = "signature", ignore = true)
	TransportReport mapTransportReport(XtaTransportReport xtaTransportReport);

	@Mapping(target = "otherAttributes", ignore = true)
	AttributedURIType mapCreateMessageIdResponse(String value);
}
