package de.ozgcloud.xta.test.app.server;

import jakarta.jws.WebService;

import org.apache.cxf.annotations.SchemaValidation;
import org.springframework.stereotype.Component;

import de.ozgcloud.xta.test.app.mapper.ExceptionMapper;
import de.ozgcloud.xta.test.app.mapper.RequestMapper;
import de.ozgcloud.xta.test.app.mapper.ResponseMapper;
import de.ozgcloud.xta.test.app.service.XtaMessageService;
import de.ozgcloud.xta.test.app.validation.RequestValidator;
import genv3.de.xoev.transport.xta.x211.InvalidMessageIDException;
import genv3.de.xoev.transport.xta.x211.MsgBoxPortType;
import genv3.de.xoev.transport.xta.x211.PermissionDeniedException;
import genv3.de.xoev.transport.xta.x211.XTAWSTechnicalProblemException;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@WebService(serviceName = "XTAService", portName = "MsgBoxPort", targetNamespace = "http://xoev.de/transport/xta/211", wsdlLocation = "classpath:wsdl/XTA.wsdl", endpointInterface = "genv3.de.xoev.transport.xta.x211.MsgBoxPortType")
@SchemaValidation(type = SchemaValidation.SchemaValidationType.BOTH)
@RequiredArgsConstructor
@Component
@Log4j2
public class MsgBoxPortImpl implements MsgBoxPortType {

	private final RequestValidator requestValidator;
	private final RequestMapper requestMapper;
	private final ExceptionMapper exceptionMapper;
	private final ResponseMapper responseMapper;
	private final XtaMessageService messageService;

	/*
	 * (non-Javadoc)
	 * @see genv3.de.xoev.transport.xta.x211.MsgBoxPortType#getMessage(genv3.eu.osci.ws.x2008.x05.transport.
	 * MsgBoxFetchRequest fetchRequest, genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier,
	 * genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData messageMetaData,
	 * genv3.eu.osci.ws.x2008.x05.transport.MsgBoxResponseType fetchResponseHeader)*
	 */
	@Override
	public genv3.de.xoev.transport.xta.x211.GenericContentContainer getMessage(
			final genv3.eu.osci.ws.x2008.x05.transport.MsgBoxFetchRequest fetchRequest,
			final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier,
			final jakarta.xml.ws.Holder<genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData> messageMetaData,
			final jakarta.xml.ws.Holder<genv3.eu.osci.ws.x2008.x05.transport.MsgBoxResponseType> fetchResponseHeader)
			throws XTAWSTechnicalProblemException, InvalidMessageIDException, PermissionDeniedException {
		log.info("Executing operation getMessage");

		var request = requestMapper.mapXtaGetMessageRequest(fetchRequest, authorIdentifier);
		requestValidator.validate(request);

		var message = messageService.getMessage(request)
				.orElseThrow(() -> exceptionMapper.createInvalidMessageIDException(request.messageId(), request.clientIdentifier().value()));

		messageMetaData.value = responseMapper.mapMessageMetaData(message.metaData());
		fetchResponseHeader.value = responseMapper.mapMsgBoxResponseTypeFromRequestId(
				messageService.createRequestId()
		);
		return responseMapper.mapGenericContentContainerFromXtaMessage(message);
	}

	/*
	 * (non-Javadoc)
	 * @see genv3.de.xoev.transport.xta.x211.MsgBoxPortType#getStatusList(genv3.eu.osci.ws.x2008.x05.transport.
	 * MsgBoxStatusListRequestType fetchRequest, genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier,
	 * genv3.eu.osci.ws.x2008.x05.transport.MsgBoxResponseType fetchResponseHeader)*
	 */
	@Override
	public genv3.eu.osci.ws.x2008.x05.transport.MsgStatusListType getStatusList(
			final genv3.eu.osci.ws.x2008.x05.transport.MsgBoxStatusListRequestType statusListRequest,
			final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier,
			final jakarta.xml.ws.Holder<genv3.eu.osci.ws.x2008.x05.transport.MsgBoxResponseType> msgBoxResponseHeader)
			throws XTAWSTechnicalProblemException, PermissionDeniedException {
		log.info("Executing operation getStatusList");

		var request = requestMapper.mapMessageMetaDataListingRequest(statusListRequest, authorIdentifier);
		requestValidator.validate(request);

		var listing = messageService.getStatusList(request);

		msgBoxResponseHeader.value = responseMapper.mapMsgBoxResponseType(listing);
		return responseMapper.mapMsgStatusListType(listing);
	}

	/*
	 * (non-Javadoc)
	 * @see genv3.de.xoev.transport.xta.x211.MsgBoxPortType#getNextMessage(genv3.eu.osci.ws.x2008.x05.transport.
	 * MsgBoxGetNextRequestType fetchRequest, genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier,
	 * genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData messageMetaData,
	 * genv3.eu.osci.ws.x2008.x05.transport.MsgBoxResponseType fetchResponseHeader)*
	 */
	@Override
	public genv3.de.xoev.transport.xta.x211.GenericContentContainer getNextMessage(
			final genv3.eu.osci.ws.x2008.x05.transport.MsgBoxGetNextRequestType fetchRequest,
			final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier,
			final jakarta.xml.ws.Holder<genv3.eu.osci.ws.x2014.x10.transport.MessageMetaData> messageMetaData,
			final jakarta.xml.ws.Holder<genv3.eu.osci.ws.x2008.x05.transport.MsgBoxResponseType> fetchResponseHeader)
			throws XTAWSTechnicalProblemException, InvalidMessageIDException, PermissionDeniedException {
		log.info("Executing operation getNextMessage");
		throw new XTAWSTechnicalProblemException("Method not supported");
	}

	/*
	 * (non-Javadoc)
	 * @see genv3.de.xoev.transport.xta.x211.MsgBoxPortType#getNextStatusList(genv3.eu.osci.ws.x2008.x05.transport.
	 * MsgBoxGetNextRequestType fetchRequest, genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier,
	 * genv3.eu.osci.ws.x2008.x05.transport.MsgBoxResponseType fetchResponseHeader)*
	 */
	@Override
	public genv3.eu.osci.ws.x2008.x05.transport.MsgStatusListType getNextStatusList(
			final genv3.eu.osci.ws.x2008.x05.transport.MsgBoxGetNextRequestType fetchRequest,
			final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier,
			final jakarta.xml.ws.Holder<genv3.eu.osci.ws.x2008.x05.transport.MsgBoxResponseType> fetchResponseHeader)
			throws XTAWSTechnicalProblemException, PermissionDeniedException {
		log.info("Executing operation getNextStatusList");
		throw new XTAWSTechnicalProblemException("Method not supported");
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * genv3.de.xoev.transport.xta.x211.MsgBoxPortType#close(genv3.eu.osci.ws.x2008.x05.transport.MsgBoxCloseRequestType
	 * fetchRequest, genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)*
	 */
	@Override
	public void close(final genv3.eu.osci.ws.x2008.x05.transport.MsgBoxCloseRequestType closeRequest,
			final genv3.eu.osci.ws.x2014.x10.transport.PartyType authorIdentifier)
			throws XTAWSTechnicalProblemException, InvalidMessageIDException, PermissionDeniedException {
		log.info("Executing operation close");
		var request = requestMapper.mapXtaCloseRequest(closeRequest, authorIdentifier);
		requestValidator.validate(request);

		if (!messageService.closeMessage(request)) {
			throw exceptionMapper.createInvalidMessageIDException(request.messageId(), request.clientIdentifier().value());
		}
	}

}
